# Purpose:  Display a dialog for editing program preferences
#

require 'fox16'
require 'SessionManager'

include Fox


module Mondrian
	class GeneralFrame < FXGroupBox
		def initialize(parent, title, options, tgt)
			super(parent, title, options)
			
			self.target = tgt
			
			@web_browser = FileChooser.new(self, "Web browser (for help):", 20)
			@web_browser.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			FXHorizontalFrame.new(self) { |frame|
				FXLabel.new(frame, "Auto-save files:")
				@timed_save = FXListBox.new(frame, nil, 0, FRAME_LINE | LISTBOX_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 150, 0)
				@timed_save.setNumVisible 6
				@timed_save << "Never"
				@timed_save << "Every minute"
				@timed_save << "Every 5 minutes"
				@timed_save << "Every 10 minutes"
				@timed_save << "Every 30 minutes"
				@timed_save << "Every hour"
				@timed_save.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			}
			
			FXHorizontalFrame.new(self) { |frame|
				@save_on_tab_switch = FXCheckButton.new(frame, "Auto-save file when tab is switched")
				@save_on_tab_switch.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			}
			
			get_preferences
		end
		
		
		def get_preferences
			session = SessionManager.instance
			
			@web_browser.text = session.restore("Preferences/Web Browser") if session.restore("Preferences/Web Browser")
			
			@timed_save.currentItem = {0=>0, 1=>1, 5=>2, 10=>3, 30=>4, 60=>5}[session.restore("Preferences/Timed Save")] if session.restore("Preferences/Timed Save")
			@save_on_tab_switch.checkState = session.restore("Preferences/Save On Tab Switch")
		end
		
		
		def apply
			session = SessionManager.instance
			
			session.store("Preferences/Web Browser", @web_browser.text)
			session.store("Preferences/Timed Save", {0=>0, 1=>1, 2=>5, 3=>10, 4=>30, 5=>60}[@timed_save.currentItem])
			session.store("Preferences/Save On Tab Switch", @save_on_tab_switch.checked?)
		end
	end



	class EditorFrame < FXGroupBox
		def initialize(parent, title, options, tgt)
			super(parent, title, options)

			self.target = tgt
			
			@line_numbers = FXCheckButton.new(self, "Display line numbers")
			@line_numbers.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			@eol_markers = FXCheckButton.new(self, "Display EOL markers")
			@eol_markers.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			f = FXHorizontalFrame.new(self)
			FXLabel.new(f, "Tab width:")
			@tab_width = FXTextField.new(f, 5, nil, 0, TEXTFIELD_INTEGER | FRAME_LINE)
			@tab_width.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			@convert_tabs = FXCheckButton.new(self, "Convert tabs to spaces")
			@convert_tabs.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }

			@show_tracker = FXCheckButton.new(self, "Show change tracker")
			@show_tracker.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			get_preferences
		end
		

		def get_preferences
			session = SessionManager.instance

			@line_numbers.checkState = session.restore("Preferences/Line Numbers")
			@eol_markers.checkState = session.restore("Preferences/EOL Markers")
			@tab_width.text = session.restore("Preferences/Tab Width").to_s if session.restore("Preferences/Tab Width")
			@convert_tabs.checkState = session.restore("Preferences/Convert Tabs")
			@show_tracker.checkState = session.restore("Preferences/Show Change Tracker")
		end
		

		def apply
			session = SessionManager.instance
			
			session.store("Preferences/Line Numbers", @line_numbers.checked?)
			print @line_numbers.checked?
			session.store("Preferences/EOL Markers", @eol_markers.checked?)
			session.store("Preferences/Convert Tabs", @convert_tabs.checked?)
			session.store("Preferences/Tab Width", @tab_width.text.to_i)
			session.store("Preferences/Show Change Tracker", @show_tracker.checked?)
		end
	end
			
			
			
	class AppearanceFrame < FXGroupBox
		def initialize(parent, title, options, tgt)
			super(parent, title, options)
			
			self.target = tgt
			
			theme_frame = FXHorizontalFrame.new(self, FRAME_NONE | LAYOUT_FILL_X)
			
			FXLabel.new(theme_frame, "Theme:")
			@theme_list = FXListBox.new(theme_frame, nil, 0, LISTBOX_NORMAL | LAYOUT_FILL_X | FRAME_LINE)
			@theme_list.setNumVisible 4
			@theme_list.connect(SEL_COMMAND) do
				refresh 
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			end
			@new_button = FXButton.new(theme_frame, "New...")
			@new_button.connect(SEL_COMMAND, method(:on_create_theme))
			@delete_button = FXButton.new(theme_frame, "Delete")
			@delete_button.connect(SEL_COMMAND, method(:on_delete_theme))
				
			style_frame = FXHorizontalFrame.new(self, FRAME_NONE | LAYOUT_FILL_X)

			FXLabel.new(style_frame, "Choose styles for:")

			@style_list = FXListBox.new(style_frame, nil, 0, LISTBOX_NORMAL | LAYOUT_FILL_X | FRAME_LINE)
			@style_list.setNumVisible 10
			@style_list.connect(SEL_COMMAND) { refresh }

			frame = FXHorizontalFrame.new(self, FRAME_NONE | LAYOUT_FILL_X)
			
			fv = FXVerticalFrame.new(frame, FRAME_NONE)
			
			@bold 			= FXCheckButton.new(fv, "Bold")
			@bold.connect(SEL_COMMAND) {
				selected_style.bold = @bold.checked?
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}

			@italic 		= FXCheckButton.new(fv, "Italic")
			@italic.connect(SEL_COMMAND) {
				selected_style.italic = @italic.checked?
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}

			@underline 	= FXCheckButton.new(fv, "Underline")
			@underline.connect(SEL_COMMAND) {
				selected_style.underline = @underline.checked?
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}

			fv2 = FXVerticalFrame.new(fv, FRAME_NONE)
			
			fh = FXHorizontalFrame.new(fv2, FRAME_NONE | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			FXLabel.new(fh, "Text Color:", nil, LAYOUT_CENTER_Y)
			@text = FXColorWell.new(fh, 0, nil, 0, LAYOUT_RIGHT | LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT | COLORWELL_OPAQUEONLY, 0, 0, 30, 30)
			@text.connect(SEL_COMMAND) {
				selected_style.foreground = sprintf("%02x%02x%02x", FXBLUEVAL(@text.rgba), FXGREENVAL(@text.rgba), FXREDVAL(@text.rgba)).hex
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			fh = FXHorizontalFrame.new(fv2, FRAME_NONE | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			@fill_label = FXLabel.new(fh, "Fill Color:", nil, LAYOUT_CENTER_Y)
			@fill = FXColorWell.new(fh, 0, nil, 0, LAYOUT_RIGHT | LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT | COLORWELL_OPAQUEONLY, 0, 0, 30, 30)
			@fill.connect(SEL_COMMAND) {
				selected_style.background = sprintf("%02x%02x%02x", FXBLUEVAL(@fill.rgba), FXGREENVAL(@fill.rgba), FXREDVAL(@fill.rgba)).hex
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			fv = FXVerticalFrame.new(frame, FRAME_NONE | LAYOUT_RIGHT | LAYOUT_FILL_X)
			
			fh = FXHorizontalFrame.new(fv, FRAME_NONE | LAYOUT_RIGHT)
			FXLabel.new(fh, "Font:")
			@fonts = FXListBox.new(fh, nil, 0, LISTBOX_NORMAL | FRAME_LINE | LAYOUT_FILL_X | LAYOUT_FIX_WIDTH, 0, 0, 150, 0)
			@fonts.setNumVisible 10
			FXFont.listFonts("").each { |desc|
				@fonts.appendItem(desc.face)
			}
			@fonts.connect(SEL_COMMAND) {
				selected_style.font = @fonts.getItemText(@fonts.currentItem)
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			fh = FXHorizontalFrame.new(fv, FRAME_NONE | LAYOUT_RIGHT)
			FXLabel.new(fh, "Size:")
			@size 			= FXTextField.new(fh, 6, nil, 0, TEXTFIELD_REAL | FRAME_LINE)
			@size.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			@size.connect(SEL_COMMAND) {
				selected_style.size = @size.text.to_i
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			get_preferences
		end
		
		def clone_themes(themes)
			result = []
			themes.each_with_index do |theme, idx|
				result[idx] = theme.built_in? ? theme : theme.clone			
			end
		end
		
		def get_preferences
			session = SessionManager.instance
			@themes = clone_themes(session.restore("Preferences/Editor Themes"))
			theme_idx = session.restore("Preferences/Editor Theme Index")			
			theme = @themes.fetch(theme_idx)
						
			@theme_list.clearItems
			@themes.each { |theme| append_theme(theme) }						
			@theme_list.setCurrentItem(theme_idx)
								
			refresh
		end
		
		def append_theme(theme)			
			type_str = theme.built_in? ? " (built in)" : " (user defined)"
			@theme_list.setCurrentItem(@theme_list.appendItem(theme.name + type_str, nil, theme))
		end
				
		def selected_theme			
			@theme_list.getItemData(@theme_list.currentItem)
		end	
		
		def selected_style
			@style_list.getItemData(@style_list.currentItem)
		end
		
		def refresh
			style_idx = @style_list.getCurrentItem
			@style_list.clearItems
			[ ["Normal text", STYLE_NORMAL_TEXT], ["Module names", STYLE_MODULE_NAME],
				["Class names", STYLE_CLASS_NAME], ["Method names", STYLE_METHOD_NAME],
				["Operators", STYLE_OPERATOR], ["Brace highlight", STYLE_BRACE_HIGHLIGHT],
				["Unmatched brace", STYLE_BRACE_BAD], ["Keywords", STYLE_KEYWORD], ["Numbers", STYLE_NUMBER],
				["Strings", STYLE_UNTERMINATED_STRING], ["Shell commands", STYLE_UNTERMINATED_SYSTEM_COMMAND],
				["Regular expressions", STYLE_UNTERMINATED_REGEXP], ["Array literals", STYLE_UNTERMINATED_ARRAY_LITERAL],
				["Comments", STYLE_COMMENT], ["System Variables", STYLE_SYSTEM_VARIABLE]].each do |pair|
			  
				@style_list.appendItem(pair[0], nil, selected_theme.styles[pair[1]])
			end
			@style_list.setCurrentItem(style_idx) if style_idx >= 0
		
			state = selected_theme.built_in? ? :disable : :enable
			[@style_list, @bold, @italic, @underline, @fill, @text, @fonts, @size, @delete_button].each do |widget|
				widget.send(state)
			end
			@bold.checkState = selected_style.bold
			@italic.checkState = selected_style.italic
			@underline.checkState = selected_style.underline
			if @fonts.findItem(selected_style.font) >= 0
				@fonts.currentItem = @fonts.findItem(selected_style.font)
			end
			@size.text = selected_style.size.to_s
			
			color = sprintf("%06x", selected_style.foreground)
			@text.rgba = FXRGB(color[4..5].hex, color[2..3].hex, color[0..1].hex)
			
			color = sprintf("%06x", selected_style.background)
			@fill.rgba = FXRGB(color[4..5].hex, color[2..3].hex, color[0..1].hex)
		end
						
		def on_create_theme(sender, message, data)
			theme_name = FXInputDialog.getString("", self, "Theme Name", "Enter a theme name:")
			return false if not theme_name
			new_theme = selected_theme.clone
			new_theme.name = theme_name			
			append_theme(new_theme)
			@themes << new_theme
			refresh
			target.handle(self, MKUINT(0, SEL_CHANGED), nil)
			true		
		end

		def on_delete_theme(sender, message, data)
			if (FXMessageBox.question(self, MBOX_YES_NO, "Confirm", "Are you sure you would like to delete the selected theme?"))
				@themes.delete_at(@theme_list.currentItem)
				@theme_list.removeItem(@theme_list.currentItem)
				refresh
				target.handle(self, MKUINT(0, SEL_CHANGED), nil)
			end
		end
		
		def apply
			session = SessionManager.instance
			
			[ STYLE_STRING,
				STYLE_REGEXP,
				STYLE_ARRAY_LITERAL,
				STYLE_SYSTEM_COMMAND ].each { |style|
					selected_theme.styles[style].copy_from(selected_theme.styles[style + 1])
			}
			selected_theme.styles[STYLE_WHITESPACE].copy_from(selected_theme.styles[STYLE_NORMAL_TEXT])					
			themes_to_store = clone_themes(@themes)
			session.store("Preferences/Editor Themes", themes_to_store)
			themes_to_store = (themes_to_store.collect { |x| x.inspect }).join("\n")
			session.store("Preferences/Editor Theme Index", @theme_list.currentItem)
		end
	end



	class ExecutionFrame < FXGroupBox
		def initialize(parent, title, options, tgt)
			super(parent, title, options)
			
			self.target = tgt
			
			@ruby_path = FileChooser.new(self, "Ruby path:", 25)
			@ruby_path.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
      @ruby_debug_path = FileChooser.new(self, "Ruby-Debug path:", 25)
			@ruby_debug_path.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
      
			
			@auto_save = FXCheckButton.new(self, "Save files automatically when run")
			@auto_save.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			@capture_stdout = FXRadioButton.new(self, "Capture program stdout")
			@capture_stdout.connect(SEL_COMMAND) { 
				@run_terminal.check = false
				@terminal.disable if @terminal; 
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			@run_terminal = FXRadioButton.new(self, "Run program in a terminal window")
			@run_terminal.connect(SEL_COMMAND) { 
				@capture_stdout.check = false
				@terminal.enable if @terminal; 
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			unless RUBY_PLATFORM =~ /win32/
				f = FXHorizontalFrame.new(self)
				FXLabel.new(f, "Terminal string:")
				@terminal = FXTextField.new(f, 25)
				@terminal.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
				@terminal.disable if @capture_stdout.checked?
			end
			
			get_preferences
		end


		def get_preferences
			session = SessionManager.instance
			
			@ruby_path.text = session.restore("Preferences/Ruby Path")
      @ruby_debug_path.text = session.restore("Preferences/Ruby-Debug Path")

			@auto_save.checkState = session.restore("Preferences/Auto Save")
			@capture_stdout.checkState = session.restore("Preferences/Capture Stdout")
			@run_terminal.checkState = (not session.restore("Preferences/Capture Stdout"))

			unless RUBY_PLATFORM =~ /win32/
				@terminal.text = session.restore("Preferences/Terminal String") if session.restore("Preferences/Terminal String")
			end
		end
		
		
		def apply
			session = SessionManager.instance
			
			session.store("Preferences/Ruby Path", @ruby_path.text)
      session.store("Preferences/Ruby-Debug Path", @ruby_debug_path.text)
			
			session.store("Preferences/Auto Save", @auto_save.checked?)
			
			session.store("Preferences/Capture Stdout", @capture_stdout.checked?)
			session.store("Preferences/Terminal String", @terminal.text) if @terminal
		end
	end


	
	#
	# PreferenceDialog:	the main preferences window
	# 
	# Notes:	calls functions directly in the owner...bad idea, should fix at some point
	#
	
	class PreferencesDialog < FXDialogBox
    def initialize(owner)
      super(owner, "Edit Preferences", DECOR_TITLE | DECOR_BORDER, 0, 0, 0, 0)
			
			@target = owner
			
			session = SessionManager.instance
			
			@need_apply = false
      
			main_frame = FXVerticalFrame.new(self, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0)
			
			content_frame = FXHorizontalFrame.new(main_frame)

			@shutter = FXShutter.new(content_frame, nil, 0, FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
    
			FXShutterItem.new(@shutter, "  General  ", nil, 0, LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT)
			FXShutterItem.new(@shutter, "  Editor  ", nil, 0, LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT)
			FXShutterItem.new(@shutter, "  Appearance  ", nil, 0, LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT)
			FXShutterItem.new(@shutter, "  Execution  ", nil, 0, LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT)
			
			switcher = FXSwitcher.new(content_frame, FRAME_NONE | LAYOUT_FIX_HEIGHT | LAYOUT_FIX_WIDTH, 0, 0, 350, 250, 0, 0, 0, 0)

			@shutter.connect(SEL_COMMAND) { |sender, message, data| switcher.current = data }
			
			@general = GeneralFrame.new(switcher, "General Options", GROUPBOX_NORMAL | FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y, self)
			@general.connect(SEL_CHANGED) { need_apply }
			@editor = EditorFrame.new(switcher, "Editor Options", GROUPBOX_NORMAL | FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y, self)
			@editor.connect(SEL_CHANGED) { need_apply }
			@appearance = AppearanceFrame.new(switcher, "Appearance Options", GROUPBOX_NORMAL | FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y, self)
			@appearance.connect(SEL_CHANGED) { need_apply }
			@execution = ExecutionFrame.new(switcher, "Execution Options", GROUPBOX_NORMAL | FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y, self)
			@execution.connect(SEL_CHANGED) { need_apply }

			# OK/cancel buttons
			
			fh = FXHorizontalFrame.new(main_frame, FRAME_NONE | LAYOUT_FILL_X)
			
			button = FXButton.new(fh, "Restore Defaults")
			button.connect(SEL_COMMAND) {
				@target.set_default_preferences(true)
				
				@general.get_preferences
				@editor.get_preferences
				@appearance.get_preferences
				@execution.get_preferences
				
				need_apply				
				apply_settings
			}
			
      frame = FXHorizontalFrame.new(fh, FRAME_NONE | LAYOUT_RIGHT, 0, 0, 0, 0, 0, 0, 0, 0)
      
      button = FXButton.new(frame, "OK", nil, nil, 0, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
      button.resize(button.font.getTextWidth("Cancel") + 25, 0)
			button.connect(SEL_COMMAND) {
				handle(self, MKUINT(FXDialogBox::ID_ACCEPT, SEL_COMMAND), nil) if apply_settings
			}
      
      @apply_button = FXButton.new(frame, "Apply", nil, nil, 0, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
      @apply_button.resize(@apply_button.font.getTextWidth("Cancel") + 25, 0)
			@apply_button.connect(SEL_COMMAND) { apply_settings }
			@apply_button.disable
			
      button = FXButton.new(frame, "Cancel", nil, self, FXDialogBox::ID_CANCEL, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
      button.resize(button.font.getTextWidth("Cancel") + 25, 0)
    end
		
		
		def need_apply
			@need_apply = true
			
			@apply_button.enable
		end
		
		
		def apply_settings
			return true unless @need_apply
			
			@general.apply
			@editor.apply
			@appearance.apply
			@execution.apply
			
			SessionManager.instance.save

			@target.apply_preferences
			
			@need_apply = false

			@apply_button.disable

			true
		end
  end
  
end