# A storage place for useful but nontrivial methods.
#
# Contributors:: Oliver Smith


module Kinshasa

    module Common

        # Converts a string to a simple regexp which matches that string.
    
        def self.regexp_from_string(string)
            /^#{Regexp.escape(string)}$/
        end
    
    
        # Returns +true+ if +path+ is a relative path.
        
        def self.is_relative?(path)
            if platform_is_windows?
                return path[0].chr == '.' || (path[0].chr != File::SEPARATOR && path[0].chr != File::ALT_SEPARATOR && path[1].chr != ':')
            else
                return path[0].chr == '.' || (path[0].chr != File::SEPARATOR && path[0].chr != File::ALT_SEPARATOR)
            end
        end
    
    
        # Returns +true+ if +path+ is an absolute path.
    
        def self.is_absolute?(path)
            not is_relative? path
        end
    
    
        # Returns +true+ if +path+ is an actual directory and not a link.
    
        def self.is_real_directory?(path)
            return File.directory?(path) && (not File.basename(path) =~ /\A\.\.?\Z/) && (not File.symlink?(path))
        end
        
        
        # Returns +true+ if +subpath+ is a subpath of +path+.
        
        def self.is_subpath?(subpath, path)
            if platform_is_windows?
                return File.expand_path(subpath).downcase.index(File.expand_path(path).downcase) == 0
            else
                return File.expand_path(subpath).index(File.expand_path(path)) == 0
            end
        end
        
        
        # Returns +true+ if we're running on a version of Windows.
        
        def self.platform_is_windows?
            return PLATFORM =~ /mswin/
        end
        
        
        # Returns the program name specified by the #! line in +path+, or nil. 
        # Only returns the program itself, not the path to it.
        
        def self.get_hash_bang_program(path)
            File.open(path, 'r') { |f|
                program = f.readline =~ /^#!(.*)$/ ? $1.split(/[\/\\]/)[-1].strip : nil
    
                return program != '' ? program : nil
            }
        end
    
    end
    
end