# project.rb
#
# Contributors:: Oliver Smith


require 'yaml'

require 'framework/common/treelist'
require 'framework/common/utilities'
require 'framework/file'
require 'framework/folder'


module Kinshasa

    # A class for managing a project directory.

    class Project < Folder
        ProjectFilename = 'project.yml'
        
        # Creates a new project at +path+, creating the folder if necessary.
        
        def self.create(path, name)
            Dir.mkdir(path) unless File.exists?(path)
            
            raise Errno::EEXIST if File.exists?(File.join(path, ProjectFilename))
            
            File.open(File.join(path, ProjectFilename), 'w') { |file|
                file.write({ 'Project' => name }.to_yaml)
            }
            
            self.new(path)
        end
        
        
        # Returns whether +abs_path+ is a project folder.

        def self.project_folder?(abs_path)
            raise ArgumentError, "Path is not absolute!" unless Common::is_absolute? abs_path

            return Common::is_real_directory?(abs_path) && File.exists?(File.join(abs_path, ProjectFilename))
        end


        # Where +path+ is a directory which contains a +project.yml+ file.
        
        def initialize(path)
            super
        
            raise Errno::ENOENT unless File.exists?(File.join(path, ProjectFilename))
            
            configure
        end
        
        
        # Configures this project from the yaml configuration file.
        
        private
        def configure
            config = YAML::load(File.new(File.join(@path, ProjectFilename), 'r'))
            
            @name = config['Project']
        end
        
    end
    
end