# workspace.rb
#
# Contributors:: Oliver Smith


require 'yaml'

require 'framework/common/utilities'
require 'framework/project'


module Kinshasa

    # A class for managing a workspace directory.

    class Workspace
        attr_reader :path, :tree
    
        def self.create(path)
            Dir.mkdir(path) unless File.exists?(path)
            
            self.new(path)
        end


        def initialize(path)
            raise Errno::ENOENT unless File.directory?(path)
        
            @path = File.expand_path(path)
            
            build_tree
        end
        
        
        def to_s
            return ':)'
        end


        # Returns whether +abs_path+ is a project folder in this workspace.

        def project_folder?(abs_path)
            raise ArgumentError, "Path (#{abs_path}) is not absolute!" unless Common::is_absolute? abs_path

            return Project.project_folder?(abs_path) && Common::is_subpath?(abs_path, @path)
        end


        # Returns the absolute names of project folders.
        
        def project_folder_paths
            folders = Dir.new(@path).entries.collect { |entry| File.join(@path, entry) }.select { |entry| project_folder? entry }
            
            folders.sort { |a, b| File.basename(a).downcase <=> File.basename(b).downcase }
        end
    

        # Loads the contained projects into a TreeList.

        private
        def build_tree
            @tree = TreeList.new(self)

            project_folder_paths.each { |project_folder_path|
                @tree.append(Project.new(project_folder_path).tree) # FIXME: create the correct project type
            }
        end

    end
    
end