# rubystring.rb
#
# Contributors:: Oliver Smith


require 'framework/tokenstring'

require 'plugins/mondrian.ruby_0.0.1/rubylexer'


module Kinshasa

    # A +TokenString+ for the ruby language.

    class RubyString < TokenString
        use_lexer RubyLexer
        
        # Returns a +RubyContext+ for +index+.
        #
        # To lex ruby we need to know both the previous token and the previous
        # non-whitespace token, because a new token's type might be affected
        # by the previous token's type.  This method fills out a +RubyContext+
        # with that information.
        
        protected
        def get_context(index)
            return RubyContext.new if index == 0
        
            i = @tokens.rindex(@encoder.match_any, index - 1)
            
            last = i ? @text[i, $&.length] : nil
            last_token = last ? @encoder.token_symbol($&) : nil
            
            if last and !lexer.in_group?(last_token, :whitespace)
                last_black = last
                last_black_token = last_token
            else
                i = @tokens.rindex(@encoder.not_whitespace, index - 1)

                last_black = i ? @text[i, $&.length] : nil
                last_black_token = i ? @encoder.token_symbol($&) : nil
            end
            
            RubyContext.new(last, last_token, last_black, last_black_token)
        end


        # Specifies where lexing should begin if a change happens at +index+.
        #
        # With ruby, we need to begin re-lexing after the previous whitespace
        # token.

        protected
        def get_lex_start(change_index)
            return 0 if change_index == 0

            lex_start = @tokens.rindex(@encoder.not_whitespace, change_index - 1)
                
            # FIXME: wtf does this do?  should it be "($& ? lex_start - $&.length : 0)"?
            while lex_start and @tokens.rindex(@encoder.not_whitespace, lex_start - 1) == lex_start - ($& ? $&.length : 0)
                lex_start -= $&.length
            end
            
            lex_start or 0
        end

    end

end
